@@ -55,9 +55,9 @@ struct MappingProtocolChannel {
 	gpointer    user_data;
 };
 
-static void     mapping_protocol_channel_do_iteration_unlocked (MappingProtocolChannel *channel,
+static gboolean mapping_protocol_channel_do_iteration_unlocked (MappingProtocolChannel *channel,
 								gboolean                blocking);
-static void     mapping_protocol_channel_do_read_iteration_unlocked (MappingProtocolChannel *channel,
+static gboolean mapping_protocol_channel_do_read_iteration_unlocked (MappingProtocolChannel *channel,
 								     gboolean                blocking);
 static void     add_write_watch                                (MappingProtocolChannel *channel);
 static gboolean handle_read                                    (GIOChannel             *source,
@@ -69,11 +69,12 @@ static gboolean handle_write                                   (GIOChannel
 
 #undef DEBUG_ENABLE
 
+
 #ifdef DEBUG_ENABLE
 #ifdef G_HAVE_ISO_VARARGS
-#  define DEBUG_PRINT(...) debug_print (__VA_ARGS__);
+#  define DEBUG_PRINT(...) profile_log (G_STRFUNC, NULL, __VA_ARGS__);
 #elif defined(G_HAVE_GNUC_VARARGS)
-#  define DEBUG_PRINT(args...) debug_print (args);
+#  define DEBUG_PRINT(args...) profile_log (G_STRFUNC, NULL, args);
 #endif
 #else
 #ifdef G_HAVE_ISO_VARARGS
@@ -84,40 +85,34 @@ static gboolean handle_write                                   (GIOChannel
 #endif
 
 #ifdef DEBUG_ENABLE
-
-static FILE *debug_out = NULL;
-
 static void
-debug_init (void)
+profile_log (const char *func,
+	     const char *note,
+	     const char *format,
+	     ...)
 {
-	const char path [50] = "mapping_protocol_debug_XXXXXX";
-	int  fd = g_file_open_tmp (path, NULL, NULL);
-	if (fd >= 0) {
-		debug_out = fdopen (fd, "a");
-	}
-}
+        va_list args;
+        char   *str;
+        char   *formatted;
 
-static void
-debug_print (const char *format, ...)
-{
-	va_list args;
-        char    buffer [1025];
+        if (format == NULL) {
+                formatted = g_strdup ("");
+        } else {
+        	va_start (args, format);
+        	formatted = g_strdup_vprintf (format, args);
+        	va_end (args);
+        }
 
-	if (debug_out == NULL) {
-		debug_init ();
-	}
+        if (func != NULL) {
+                str = g_strdup_printf ("MARK: %s %s: %s %s", g_get_prgname(), func, note ? note : "", formatted);
+        } else {
+                str = g_strdup_printf ("MARK: %s: %s %s", g_get_prgname(), note ? note : "", formatted);
+        }
 
-	va_start (args, format);
-        g_vsnprintf (buffer, 1024, format, args);
-	va_end (args);
+        g_free (formatted);
 
-        fprintf ((debug_out ? debug_out : stderr),
-		 "[%p] %s",
-                 g_thread_self (), buffer);
-
-	if (debug_out) {
-		fflush (debug_out);
-	}
+        g_access (str, F_OK);
+        g_free (str);
 }
 #endif
 
@@ -200,7 +195,7 @@ mapping_protocol_message_new_reply (MappingProtocolMessage *message)
 	serial = mapping_protocol_message_get_serial (message);
 	mapping_protocol_message_set_serial (reply, serial);
 
-	DEBUG_PRINT ("Creating new reply for serial:%u\n", serial);
+	DEBUG_PRINT ("Creating new reply for serial:%u", serial);
 
 	return reply;
 }
@@ -877,7 +872,7 @@ mapping_protocol_channel_dispatch_unlocked (MappingProtocolChannel *channel)
 
 	status = G_IO_STATUS_NORMAL;
 	if (channel->write_buffer->len > MINIMUM_MESSAGE_SIZE) {
-		DEBUG_PRINT ("Dispatching messages\n");
+		DEBUG_PRINT ("Dispatching messages");
 
 		error = NULL;
 		status = g_io_channel_write_chars (channel->iochannel,
@@ -889,7 +884,7 @@ mapping_protocol_channel_dispatch_unlocked (MappingProtocolChannel *channel)
 		channel->write_buffer = g_string_erase (channel->write_buffer, 0, bytes);
 
 		if (status != G_IO_STATUS_NORMAL) {
-			DEBUG_PRINT ("Write error: %s\n", error->message);
+			DEBUG_PRINT ("Write error: %s", error->message);
 			g_error_free (error);
 		} else {
 			g_io_channel_flush (channel->iochannel, NULL);
@@ -903,7 +898,7 @@ static void
 mapping_protocol_channel_queue_write_unlocked (MappingProtocolChannel *channel,
 					       GString                *str)
 {
-	DEBUG_PRINT ("Adding string to write queue len:%d\n", str->len);
+	DEBUG_PRINT ("Adding string to write queue len:%d", str->len);
 	g_string_append_len (channel->write_buffer, str->str, str->len);
 
 	/* watch for time to write */
@@ -968,7 +963,7 @@ lookup_reply_for_serial (MappingProtocolChannel *channel,
 {
 	MappingProtocolMessage *reply;
 
-	DEBUG_PRINT ("Looking for reply for %u\n", serial);
+	DEBUG_PRINT ("Looking for reply for %u", serial);
 
 	reply = g_hash_table_lookup (channel->replies, GUINT_TO_POINTER (serial));
 
@@ -980,7 +975,7 @@ insert_reply_for_serial (MappingProtocolChannel *channel,
 			 guint32                 serial,
 			 MappingProtocolMessage *message)
 {
-	DEBUG_PRINT ("Inserting reply for serial %u\n", serial);
+	DEBUG_PRINT ("Inserting reply for serial %u", serial);
 
 	/* hash takes ownership of ref */
 	g_hash_table_insert (channel->replies, GUINT_TO_POINTER (serial), message);
@@ -990,7 +985,7 @@ static void
 remove_reply_for_serial (MappingProtocolChannel *channel,
 			 guint32                 serial)
 {
-	DEBUG_PRINT ("Removing reply for serial %u\n", serial);
+	DEBUG_PRINT ("Removing reply for serial %u", serial);
 	g_hash_table_remove (channel->replies, GUINT_TO_POINTER (serial));
 }
 
@@ -1042,38 +1037,38 @@ block_for_reply (MappingProtocolChannel  *channel,
 
 	G_LOCK (channel_lock);
 
-	/* ensure the message was sent */
-	mapping_protocol_channel_dispatch_unlocked (channel);
-
 	remove_read_watch (channel);
 
+	/* ensure the message was sent */
+	res = mapping_protocol_channel_dispatch_unlocked (channel);
+	if (! res) {
+		goto out;
+	}
+
 	/* wait for reply */
 	serial = mapping_protocol_message_get_serial (message);
 
-	DEBUG_PRINT ("Send message %u, waiting for reply\n", serial);
+	DEBUG_PRINT ("Send message %u, waiting for reply", serial);
 
-	r = NULL;
+	r = lookup_reply_for_serial (channel, serial);
 	count = 1;
-	while (count < 100000) {
+	while (r == NULL && count < 100000) {
 		mapping_protocol_channel_do_read_iteration_unlocked (channel, TRUE);
 
 		/* FIXME: check connection */
 
 		r = lookup_reply_for_serial (channel, serial);
-		if (r != NULL) {
-			break;
-		}
 
 		count++;
 	}
 
 	if (r != NULL) {
-		DEBUG_PRINT ("Got reply for message %u\n", serial);
+		DEBUG_PRINT ("Got reply for message %u", serial);
 
 		mapping_protocol_message_ref (r);
 		res = TRUE;
 	} else {
-		DEBUG_PRINT ("WARNING: Did not receive reply for message %u\n", serial);
+		DEBUG_PRINT ("WARNING: Did not receive reply for message %u", serial);
 		res = FALSE;
 	}
 
@@ -1083,10 +1078,11 @@ block_for_reply (MappingProtocolChannel  *channel,
 
 	remove_reply_for_serial (channel, serial);
 
+	/* FIXME: handle result? */
 	mapping_protocol_channel_do_iteration_unlocked (channel, FALSE);
 
 	add_read_watch (channel);
-
+ out:
 	G_UNLOCK (channel_lock);
 
 	return res;
@@ -1171,7 +1167,7 @@ decode_reply (MappingProtocolChannel  *channel,
 	return res;
 }
 
-static void
+static gboolean
 mapping_protocol_channel_fill_read_buffer_unlocked (MappingProtocolChannel *channel,
 						    gboolean                blocking)
 {
@@ -1180,7 +1176,7 @@ mapping_protocol_channel_fill_read_buffer_unlocked (MappingProtocolChannel *chan
 	gsize     length;
 	gsize     count;
 
-	DEBUG_PRINT ("Reading... \n");
+	DEBUG_PRINT ("Reading... ");
 	count = sizeof (data);
 
 	if (blocking) {
@@ -1189,16 +1185,18 @@ mapping_protocol_channel_fill_read_buffer_unlocked (MappingProtocolChannel *chan
 
 	status = g_io_channel_read_chars (channel->iochannel, data, count, &length, NULL);
 
-	if (status != G_IO_STATUS_NORMAL) {
-		return;
+	if (status == G_IO_STATUS_EOF) {
+		return FALSE;
 	}
 
 	if (length > 0) {
 		g_string_append_len (channel->read_buffer, data, length);
-		DEBUG_PRINT ("Filling read buffer: length %d\n", length);
+		DEBUG_PRINT ("Filling read buffer: length %d", length);
 	} else {
-		DEBUG_PRINT ("No data read into buffer\n");
+		DEBUG_PRINT ("No data read into buffer");
 	}
+
+	return TRUE;
 }
 
 static void
@@ -1212,7 +1210,7 @@ mapping_protocol_channel_queue_messages_unlocked (MappingProtocolChannel *channe
 		return;
 	}
 
-	DEBUG_PRINT ("Queuing messages...\n");
+	DEBUG_PRINT ("Queuing messages...");
 
 	ok = TRUE;
 	while (ok) {
@@ -1224,7 +1222,7 @@ mapping_protocol_channel_queue_messages_unlocked (MappingProtocolChannel *channe
 			break;
 		}
 
-		DEBUG_PRINT ("Processing message of type: %c\n", c);
+		DEBUG_PRINT ("Processing message of type: %c", c);
 
 		message = NULL;
 		switch (c) {
@@ -1247,7 +1245,7 @@ mapping_protocol_channel_queue_messages_unlocked (MappingProtocolChannel *channe
 		}
 
 		if (message != NULL) {
-			DEBUG_PRINT ("Queuing message of type: %c\n", c);
+			DEBUG_PRINT ("Queuing message of type: %c", c);
 			/* list takes ownership of ref */
 			channel->message_queue = g_list_append (channel->message_queue, message);
 		}
@@ -1262,7 +1260,7 @@ mapping_protocol_channel_deliver_messages_unlocked (MappingProtocolChannel *chan
 
 		message = channel->message_queue->data;
 
-		DEBUG_PRINT ("Delivering message of type: %d\n", message->type);
+		DEBUG_PRINT ("Delivering message of type: %d", message->type);
 
 		if (channel->message_handler != NULL) {
 			channel->message_handler (channel, message, channel->user_data);
@@ -1274,40 +1272,61 @@ mapping_protocol_channel_deliver_messages_unlocked (MappingProtocolChannel *chan
 	}
 }
 
-static void
+static gboolean
 mapping_protocol_channel_do_read_iteration_unlocked (MappingProtocolChannel *channel,
 						     gboolean                blocking)
 {
-	mapping_protocol_channel_fill_read_buffer_unlocked (channel, blocking);
-	mapping_protocol_channel_queue_messages_unlocked (channel);
+	gboolean res;
+
+	res = mapping_protocol_channel_fill_read_buffer_unlocked (channel, blocking);
+	if (res) {
+		mapping_protocol_channel_queue_messages_unlocked (channel);
+	}
+
+	return res;
 }
 
-static void
+static gboolean
 mapping_protocol_channel_do_iteration_unlocked (MappingProtocolChannel *channel,
 						gboolean                blocking)
 {
-	mapping_protocol_channel_do_read_iteration_unlocked (channel, blocking);
-	mapping_protocol_channel_deliver_messages_unlocked (channel);
+	gboolean res;
+
+	res = mapping_protocol_channel_do_read_iteration_unlocked (channel, blocking);
+
+	if (res) {
+		mapping_protocol_channel_deliver_messages_unlocked (channel);
+	}
+
+	return res;
 }
 
-static void
+static gboolean
 mapping_protocol_channel_do_iteration (MappingProtocolChannel *channel)
 {
-	DEBUG_PRINT ("Do iteration\n");
+	gboolean res;
+
+	DEBUG_PRINT ("Do iteration");
 
 	G_LOCK (channel_lock);
-	mapping_protocol_channel_do_iteration_unlocked (channel, FALSE);
+	res = mapping_protocol_channel_do_iteration_unlocked (channel, FALSE);
 	G_UNLOCK (channel_lock);
+
+	return res;
 }
 
-static void
+static gboolean
 mapping_protocol_channel_dispatch (MappingProtocolChannel *channel)
 {
-	DEBUG_PRINT ("Do dispatch\n");
+	gboolean res;
+
+	DEBUG_PRINT ("Do dispatch");
 
 	G_LOCK (channel_lock);
-	mapping_protocol_channel_dispatch_unlocked (channel);
+	res = mapping_protocol_channel_dispatch_unlocked (channel);
 	G_UNLOCK (channel_lock);
+
+	return res;
 }
 
 static gboolean
@@ -1318,6 +1337,7 @@ handle_write (GIOChannel             *source,
 	gboolean res;
 
         if (condition & G_IO_OUT) {
+		/* FIXME: handle result? */
 		mapping_protocol_channel_dispatch (channel);
 	}
 
@@ -1337,12 +1357,15 @@ handle_read (GIOChannel             *source,
 	res = TRUE;
 
         if (condition & G_IO_IN) {
-		mapping_protocol_channel_do_iteration (channel);
+		res = mapping_protocol_channel_do_iteration (channel);
 	}
 
 	if (condition & G_IO_HUP) {
-		channel->read_watch_id = 0;
 		res = FALSE;
+	}
+
+	if (! res) {
+		channel->read_watch_id = 0;
 		/* send a NULL message to signal error */
 		if (channel->message_handler != NULL) {
 			channel->message_handler (channel, NULL, channel->user_data);
