@@ -366,7 +366,7 @@ update_speeds_combobox (NautilusBurnDrive *drive,
 			switch (type) {
 			case NAUTILUS_BURN_MEDIA_TYPE_CDR:
 			case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
-				name = g_strdup_printf ("%d\303\227",
+				name = g_strdup_printf ("%.1f\303\227",
 							NAUTILUS_BURN_DRIVE_CD_SPEED (write_speeds[i]));
 				break;
 			case NAUTILUS_BURN_MEDIA_TYPE_DVDR:
@@ -375,7 +375,7 @@ update_speeds_combobox (NautilusBurnDrive *drive,
 			case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R:
 			case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
 			case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R_DL:
-				name = g_strdup_printf ("%d\303\227",
+				name = g_strdup_printf ("%.1f\303\227",
 							NAUTILUS_BURN_DRIVE_DVD_SPEED (write_speeds[i]));
 				break;
 			default:
@@ -383,7 +383,7 @@ update_speeds_combobox (NautilusBurnDrive *drive,
 			}
 
 		} else {
-			name = g_strdup_printf ("CD %d\303\227, DVD %d\303\227",
+			name = g_strdup_printf ("CD: %.1f\303\227  DVD: %.1f\303\227",
 						NAUTILUS_BURN_DRIVE_CD_SPEED (write_speeds[i]),
 						NAUTILUS_BURN_DRIVE_DVD_SPEED (write_speeds[i]));
 		}
@@ -398,11 +398,11 @@ update_speeds_combobox (NautilusBurnDrive *drive,
 			is_dvd = (type & NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER)
 				|| (type & NAUTILUS_BURN_DRIVE_TYPE_DVD_RAM_RECORDER);
 			if (is_dvd) {
-				name = g_strdup_printf ("CD %d\303\227, DVD %d\303\227",
+				name = g_strdup_printf ("CD %.1f\303\227, DVD %.1f\303\227",
 							NAUTILUS_BURN_DRIVE_CD_SPEED (write_speeds[i]),
 							NAUTILUS_BURN_DRIVE_DVD_SPEED (write_speeds[i]));
 			} else {
-				name = g_strdup_printf ("%d\303\227",
+				name = g_strdup_printf ("%.1f\303\227",
 							NAUTILUS_BURN_DRIVE_CD_SPEED (write_speeds[i]));
 			}
 		}
@@ -528,6 +528,7 @@ speed_changed (GtkComboBox        *widget,
 	       NcbSelectionDialog *dialog)
 {
 	int speed;
+
 	ncb_selection_dialog_get_speed (dialog, &speed);
 	ncb_selection_set_speed (dialog->priv->selection, speed);
 }
@@ -638,7 +639,7 @@ ncb_selection_dialog_init (NcbSelectionDialog *dialog)
 
 	init_dialog (dialog);
 
-	gtk_window_set_icon_name (GTK_WINDOW (dialog), "gnome-dev-cdrom");
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), "nautilus-cd-burner");
 
 	/* make insensitive until fill_dialog finishes */
 	gtk_widget_set_sensitive (GTK_WIDGET (dialog), FALSE);
